@@ -28,7 +28,9 @@ mod image;
 pub use image::*;
 mod draw;
 pub use draw::*;
+mod date;
 mod hands;
+mod moonphase;
 
 pub const fn from_u8_rgb(r: u8, g: u8, b: u8) -> u32 {
     let (r, g, b) = (r as u32, g as u32, b as u32);
@@ -37,6 +39,10 @@ pub const fn from_u8_rgb(r: u8, g: u8, b: u8) -> u32 {
 pub(crate) const AZURE_BLUE: u32 = from_u8_rgb(0, 127, 255);
 pub(crate) const _RED: u32 = from_u8_rgb(157, 37, 10);
 pub(crate) const WHITE: u32 = from_u8_rgb(255, 255, 255);
+pub(crate) const MOON: u32 = from_u8_rgb(0xd4, 0xc6, 0xa8);
+pub(crate) const MOONDARK: u32 = from_u8_rgb(0x59, 0x53, 0x45);
+pub(crate) const SUN: u32 = from_u8_rgb(0xff, 0xeb, 0x3b);
+pub(crate) const SUNDARK: u32 = from_u8_rgb(0xff, 0xa3, 0x01);
 pub(crate) const BLACK: u32 = 0;
 
 #[derive(Clone, Copy)]
@@ -148,11 +154,29 @@ const TAIL: Bitmap<'static> = Bitmap {
     y_offset: TAIL_OFFSET_Y,
 };
 
+include!("octaveback.xbm.rs");
+const OCTAVEBACK: Bitmap<'static> = Bitmap {
+    bits: OCTAVEBACK_BITS,
+    width: OCTAVEBACK_WIDTH,
+    height: OCTAVEBACK_HEIGHT,
+    x_offset: 0,
+    y_offset: 0,
+};
+
+include!("octavetail.xbm.rs");
+const OCTAVETAIL: Bitmap<'static> = Bitmap {
+    bits: OCTAVETAIL_BITS,
+    width: OCTAVETAIL_WIDTH,
+    height: OCTAVETAIL_HEIGHT,
+    x_offset: 34,
+    y_offset: 238,
+};
+
 const NUM_TAILS: usize = 10;
 
 const N_TAIL_PTS: usize = 7;
 
-fn create_eye_pixmap(t: f64) -> Image {
+fn create_eye_pixmap(t: f64, dog: bool) -> Image {
     macro_rules! tr {
         ($cond:expr ,? $then:expr ,: $else:expr) => {
             if $cond {
@@ -166,8 +190,8 @@ fn create_eye_pixmap(t: f64) -> Image {
         bytes: vec![WHITE; 30 * 60],
         width: 60,
         height: 30,
-        x_offset: 47,
-        y_offset: 30,
+        x_offset: 47 + if dog { 5 } else { 0 },
+        y_offset: 30 + if dog { 10 } else { 0 },
     };
 
     //ret.draw_outline();
@@ -241,6 +265,48 @@ fn create_eye_pixmap(t: f64) -> Image {
     ret
 }
 
+fn dog_tail(t: f64) -> Image {
+    /*  Pendulum parameters */
+    let s: f64;
+    let c: f64;
+    const A: f64 = 0.4;
+    let omega: f64 = 1.0;
+    let phi: f64 = 3.0 * FRAC_PI_2;
+    let angle: f64;
+
+    /*
+     *  Compute pendulum function.
+     */
+    angle = A * f64::sin(omega * t + phi);
+    s = f64::sin(angle);
+    c = f64::cos(angle);
+
+    let mut buf = Image::new(
+        OCTAVETAIL_WIDTH,
+        OCTAVETAIL_HEIGHT + 4,
+        OCTAVETAIL.x_offset,
+        OCTAVETAIL.y_offset,
+    );
+    /*
+     *  Rotate the center tail about its origin by "angle" degrees.
+     */
+    let t = bits_to_bytes(OCTAVETAIL_BITS, OCTAVETAIL_WIDTH);
+    let center_point = (40, 0);
+    for y in 0..OCTAVETAIL_HEIGHT {
+        for x in 0..OCTAVETAIL_WIDTH {
+            if t[y * OCTAVETAIL_WIDTH + x] == BLACK {
+                let x = (x as i64 - center_point.0) as f64;
+                let y = (y as i64 - center_point.1) as f64;
+                let xr = (x * c - y * s) as i64 + center_point.0;
+                let yr = (x * s + y * c) as i64 + center_point.1;
+                buf.plot(xr, yr);
+                buf.plot(xr + 1, yr);
+            }
+        }
+    }
+    buf
+}
+
 fn create_tail_image(t: f64) -> Image {
     /*  Pendulum parameters */
     let sin_theta: f64;
@@ -411,7 +477,7 @@ fn create_tail_image_hook(t: f64) -> Image {
     Image::from(Bitmap { bits: &ret, ..TAIL })
 }
 
-const HELP: &str = r#"Usage: kitkat [--hook|--crazy|--offset OFFSET|--borderless|--resize]
+const HELP: &str = r#"Usage: kitkat [--hook|--crazy|--offset OFFSET|--borderless|--resize|--sunmoon|--moon|--date]
 
 Displays a kit kat clock with the system time, or the system time with given offset if the --offset
 argument is provided.
@@ -422,6 +488,10 @@ argument is provided.
                              offset will be used)
       --borderless
       --resize
+      --sunmoon              show sun or moon phase depending on the hour
+      --moon                 show only moon phase
+      --date                 show month date
+      --dog                  show an italian greyhound named Gaius Octavius Maximus instead of a cat
 
       OFFSET format is [+-]{0,1}\d\d:\d\d, e.g: 02:00 or -03:45 or +00:00
 "#;
@@ -432,7 +502,7 @@ fn main() {
     unsafe {
         libc::localtime_r(&time as *const _, tm.as_mut_ptr());
     }
-    let tm = unsafe { tm.assume_init() };
+    let mut tm = unsafe { tm.assume_init() };
     let args = std::env::args().skip(1).collect::<Vec<String>>();
     if !args.is_empty() && args.iter().any(|s| s == "--help") {
         if args.iter().any(|s| s != "--help") {
@@ -446,6 +516,14 @@ fn main() {
 
     let borderless = !args.is_empty() && args.iter().any(|s| s == "--borderless");
     let resize = !args.is_empty() && args.iter().any(|s| s == "--resize");
+    let sunmoon = !args.is_empty() && args.iter().any(|s| s == "--sunmoon");
+    let moon = !args.is_empty() && args.iter().any(|s| s == "--moon");
+    if sunmoon && moon {
+        eprintln!("ERROR: You can't use both --sunmoon and --moon.");
+        return;
+    }
+    let show_date = !args.is_empty() && args.iter().any(|s| s == "--date");
+    let dog = !args.is_empty() && args.iter().any(|s| s == "--dog");
 
     let mut tail_kind: fn(_) -> _ = create_tail_image;
     let crazy: usize = args.iter().filter(|s| *s == "--crazy").count();
@@ -498,8 +576,12 @@ fn main() {
     let mut eyes_frames: Vec<Image> = Vec::with_capacity(NUM_TAILS);
 
     for i in 0..NUM_TAILS {
-        tails_frames.push(tail_kind(i as f64 * PI / (NUM_TAILS as f64)));
-        eyes_frames.push(create_eye_pixmap(i as f64 * PI / (NUM_TAILS as f64)));
+        if dog {
+            tails_frames.push(dog_tail(i as f64 * PI / (NUM_TAILS as f64)));
+        } else {
+            tails_frames.push(tail_kind(i as f64 * PI / (NUM_TAILS as f64)));
+        }
+        eyes_frames.push(create_eye_pixmap(i as f64 * PI / (NUM_TAILS as f64), dog));
     }
 
     let mut window = Window::new(
@@ -523,12 +605,22 @@ fn main() {
 
     let catwhite = Image::from(CATWHITE);
     catwhite.draw(&mut buffer, WHITE, Some(WHITE));
-    let catback = Image::from(CATBACK);
+    let catback = if dog {
+        Image::from(OCTAVEBACK)
+    } else {
+        Image::from(CATBACK)
+    };
     catback.draw(&mut buffer, BLACK, None);
-    let cattie = Image::from(CATTIE);
-    cattie.draw(&mut buffer, AZURE_BLUE, None);
-    let tail = Image::from(TAIL);
-    tail.draw(&mut buffer, BLACK, None);
+    if !dog {
+        let cattie = Image::from(CATTIE);
+        cattie.draw(&mut buffer, AZURE_BLUE, None);
+    }
+    let tail = if dog {
+        Image::from(OCTAVETAIL)
+    } else {
+        Image::from(TAIL)
+    };
+    tail.draw(&mut buffer, WHITE, None);
     let eyes = Image::from(EYES);
     eyes.draw(&mut buffer, BLACK, None);
 
@@ -598,7 +690,7 @@ fn main() {
         HOUR_HAND_WIDTH,
         HOUR_HAND_HEIGHT,
         -5,
-        (hour as f64) / 12.0,
+        0.5 * (((hour as usize % 12) * 60 + (minutes as usize)) as f64) / 360.,
     );
     hour_hand.draw(&mut buffer, BLACK, None);
     hands::draw_second(
@@ -614,12 +706,22 @@ fn main() {
         MINUTE_HAND_WIDTH,
         MINUTE_HAND_HEIGHT,
         -5,
-        (minutes as f64) / 60.0,
+        (6. * (minutes as f64)) / 360.,
     );
     minute_hand.draw(&mut buffer, BLACK, None);
+
+    let sun: Image = moonphase::sun();
+    let sun_bg: Image = moonphase::sun_background();
+
+    let moon_corners: Image = moonphase::corner_fill();
+    let full_moon: Image = moonphase::MoonPosition::FullMoon.into();
+    let moon_phase: Image = moonphase::phase(moonphase::position(None)).into();
+
+    let mut date: Image = date::make_date(tm.tm_mday as i64);
+
     while window.is_open() && !window.is_key_down(Key::Escape) && !window.is_key_down(Key::Q) {
         let cur_tail = &tails_frames[i];
-        tail.draw(&mut buffer, BLACK, Some(WHITE));
+        tails_frames[prev_i].draw(&mut buffer, WHITE, None);
         cur_tail.draw(&mut buffer, BLACK, None);
         let cur_eyes = &eyes_frames[i];
         eyes_frames[prev_i].draw(&mut buffer, WHITE, None);
@@ -661,6 +763,13 @@ fn main() {
                 if hour == 24 {
                     hour = 0;
                 }
+                if show_date && hour <= 2 {
+                    unsafe {
+                        let time = libc::time(std::ptr::null_mut());
+                        libc::localtime_r(&time as *const _, &mut tm as *mut _);
+                    }
+                    date = date::make_date(tm.tm_mday as i64);
+                }
             }
             minute_hand.draw(&mut buffer, WHITE, None);
             minute_hand.clear();
@@ -669,25 +778,41 @@ fn main() {
                 MINUTE_HAND_WIDTH,
                 MINUTE_HAND_HEIGHT,
                 -5,
-                (minutes as f64) / 60.0,
+                (6. * (minutes as f64)) / 360.,
             );
         }
         minute_hand.draw(&mut buffer, BLACK, None);
         //blank_face.draw(&mut buffer, WHITE, Some(WHITE));
         hour_hand.draw(&mut buffer, WHITE, None);
         hour_hand.clear();
+        if show_date {
+            date.draw(&mut buffer, BLACK, None);
+        }
         hands::draw_second(
             &mut hour_hand,
             HOUR_HAND_WIDTH,
             HOUR_HAND_HEIGHT,
             -5,
-            (hour as f64) / 12.0,
+            0.5 * (((hour as usize % 12) * 60 + (minutes as usize)) as f64) / 360.,
         );
         hour_hand.draw(&mut buffer, BLACK, None);
-        //}
+
+        if moon || sunmoon {
+            // FIXME: use the https://en.wikipedia.org/wiki/Sunrise_equation to calc sunrise times
+            // by having user provide latitude
+            if moon || hour < 8 || hour > 18 {
+                moon_corners.draw(&mut buffer, BLACK, Some(WHITE));
+                full_moon.draw(&mut buffer, MOONDARK, None);
+                moon_phase.draw(&mut buffer, MOON, None);
+            } else if sunmoon {
+                sun_bg.draw(&mut buffer, SUNDARK, None);
+                sun.draw(&mut buffer, SUN, None);
+            }
+        }
 
         if crazy > 0 {
-            for _ in i..(crazy + i) {
+            let r = i;
+            for _ in r..(crazy + r) {
                 if up {
                     if i + 1 == tails_frames.len() {
                         up = false;
